import {Component} from '@angular/core';
import {BackendService, ScrumTask, ScrumUserstory} from '../services/backend.service';
import {NgbModal} from '@ng-bootstrap/ng-bootstrap';
import {TableComponentBase} from '../services/table-component.base';
import {getNumberForPriority} from '../services/sorting.service';
import {UserstoryFormComponent} from '../userstory-form/userstory-form.component';
import {ActivatedRoute, ParamMap, Router} from '@angular/router';

@Component({
  selector: 'app-userstory-table',
  templateUrl: './userstory-table.component.html',
  styleUrls: ['./userstory-table.component.css']
})
export class UserstoryTableComponent extends TableComponentBase<ScrumUserstory> {
  public tasks: ScrumTask[] = [];
  public filterPriority: string | null = null;
  public highlightId: number;

  public get filteredItems() {
    return this.items.filter(task => this.filterPriority === null || task.priority === this.filterPriority);
  }

  constructor(
    private backendService: BackendService, private modalService: NgbModal,
    private route: ActivatedRoute, private router: Router
  ) {
    super();

    this.applyFilterParameters(this.route.snapshot.paramMap);
    this.route.paramMap.subscribe(map => this.applyFilterParameters(map));

    backendService.getUserstories().subscribe(response => {
      if (response.status > 399) {
        alert('Fehler');
      } else {
        this.items.push(...response.body);
      }
    });
    backendService.getTasks().subscribe(response => {
      if (response.status > 399) {
        alert('Fehler');
      } else {
        this.tasks.push(...response.body);
      }
    });
  }

  private applyFilterParameters(params: ParamMap) {
    if (params.has('id')) {
      this.highlightId = parseInt(params.get('id'));
    }
  }

  public deleteUserstory(userstory: ScrumUserstory) {
    this.backendService.deleteUserstory(userstory).subscribe(response => {
      if (response.status > 399) {
        alert('Fehler');
      }
    });
    const index = this.items.indexOf(userstory);
    if (index !== -1) {
      this.items.splice(index, 1);
    }
  }

  public openUserstoryForm(editUserstory?: ScrumUserstory) {
    const modalRef = this.modalService.open(UserstoryFormComponent, {
      backdrop: 'static',
      keyboard: true,
    });
    if (editUserstory === null) {
      modalRef.result.then(result => {
        this.items.push(result);
      });
    }
    modalRef.componentInstance.userstory = editUserstory;
  }

  public getNumberOfTasks(userstory: ScrumUserstory) {
    return this.tasks.filter(t => t.userstoryid === userstory.id).length;
  }

  public sortById() {
    this.doNumericSort('id', us => us.id);
  }

  public sortByTitle() {
    this.doStringSort('title', us => us.title);
  }

  public sortByPrio() {
    this.doNumericSort('priority', us => getNumberForPriority(us.priority));
  }

  public sortByTasks() {
    this.doNumericSort('tasks', us => this.getNumberOfTasks(us));
  }

  sortByStatus() {
    this.doStringSort('title', us => us.title);
  }
  sortByCategory() {
    this.doStringSort('title', us => us.title);
  }
}
